/**************************************************************************/
/*   AIC.H                                                                */
/*                                                                        */
/*   TMS320C3x - AIC DRIVER HEADER FILE                                   */
/*      :TMS320C3x CODE                                                   */
/*                                                                        */
/*   Leor Brenman - Applications Engineer                                 */
/*   (C) 1991 TEXAS INSTRUMENTS, HOUSTON                                  */
/**************************************************************************/
/*                                                                        */
/*   DEVICE INTERFACE:                                                    */
/*                                                                        */
/*                                                                        */
/*     TMS320C30 DSP                   TLC3204x AIC                       */
/*                                                                        */
/*         TCLK0  --------->---------- MSTR CLK                           */
/*                                                                        */
/*         FSX0   ---------<---------- FSX                                */
/*                                                                        */
/*         DX0    --------->---------- DX                                 */
/*                                                                        */
/*         FSR0   ---------<---------- FSR                                */
/*                                                                        */
/*         XF0    --------->---------- RESET                              */
/*                                                                        */
/*         DR0    ---------<---------- DR                                 */
/*                                                                        */
/*         CLKR0  ---------<---------- SHIFT CLK                          */
/*         CLKR0  ____|                                                   */
/*                                                                        */
/*                                                                        */
/**************************************************************************/
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <c30_per.h>
#include <commdrvr.h>

/*========================================================================*/
/* MACROS *===============================================================*/
/*========================================================================*/
#define BLOCK_SIZE     1024             /* Buffer size                    */
#define ASYNC          FALSE            /* Determines sync vs. async mode */
#define TLC32046       TRUE             /* Determines TLC device          */
#define MSTR_CLOCK     TRUE             /* User timer for master clock    */
#define SER_NUM        SERIAL_PORT_ZERO
#define TIMER_NUM      TIMER_ZERO
#define XF_NUM         0

#define INIT_ARRAYS    init_arrays(t_buffer,r_buffer)
#define WAIT_BUFFERS   while(!buffer_rcvd || !buffer_xmtd);
#define RESET_FLAGS    buffer_rcvd = buffer_xmtd = FALSE

#if XF_NUM
#define RESET_AIC      asm("  AND 2fh,IOF"); asm(" OR  20h,IOF")
#define UN_RESET_AIC   asm(" OR  60h,IOF")
#else
#define RESET_AIC      asm("  AND 0f2h,IOF"); asm(" OR  02h,IOF")
#define UN_RESET_AIC   asm(" OR  06h,IOF")
#endif

#define WAIT(A)        for(i=0;i<A;i++);

/*------------------------------------------------------------------------*/
/* AIC PRIMARY COMMAND MACROS						  */
/*------------------------------------------------------------------------*/
#define STANDARD       0
#if ASYNC
#define ADD	       1
#define SUBTRACT       2
#endif
#define SECONDARY_REQ  3
/*------------------------------------------------------------------------*/
/* AIC VOLTAGE INPUT CONTROL MACROS                                       */
/*------------------------------------------------------------------------*/
#define THREE_V    1
#define LINE_V     2

/*======================================================================*/
/*   AIC COMMAND WORD BITFIELD ENCODING STRUCTURES (TLC32046)		*/
/*======================================================================*/
/*   AIC PRIMARY COMMAND						*/
/*======================================================================*/
typedef union
{
    unsigned int _intval;
    struct {
      unsigned int command :2;     /* COMMAND BITS                      */
        signed int data    :14;    /* DATA                              */
      unsigned int unused  :16;
    } _bitval;
} AIC_PRIMARY;

/*======================================================================*/
/*   AIC SECONDARY COMMANDS						*/
/*======================================================================*/
typedef struct
{
    unsigned int command :2;     /* COMMAND BITS --- SHOULD BE SET TO 00*/
    unsigned int ra      :5;     /* RECEIVE COUNTER A LOAD VALUE        */
    unsigned int d_78    :2;     /* UNUSED - DON'T CARES                */
    unsigned int ta      :5;     /* TRANSMIT COUNTER A LOAD VALUE       */
    unsigned int d_ef    :2;     /* UNUSED - DON'T CARES                */
} AIC_COMMAND_0;

typedef struct
{
    unsigned int command  :2;    /* COMMAND BITS --- SHOULD BE SET TO 01*/
      signed int ra_prime :6;    /* RECEIVE COUNTER DELTA A' LOAD VALUE */
    unsigned int d_8      :1;    /* UNUSED - DON'T CARES                */
      signed int ta_prime :6;    /* RECEIVE COUNTER DELTA A' LOAD VALUE */
    unsigned int d_f      :1;    /* UNUSED - DON'T CARES                */
} AIC_COMMAND_1;

typedef struct
{
    unsigned int command :2;     /* COMMAND BITS --- SHOULD BE SET TO 10*/
    unsigned int rb      :6;     /* RECEIVE COUNTER B LOAD VALUE        */
    unsigned int d_8     :1;     /* UNUSED - DON'T CARES                */
    unsigned int tb      :6;     /* TRANSMIT COUNTER B LOAD VALUE       */
    unsigned int d_f     :1;     /* UNUSED - DON'T CARES                */
} AIC_COMMAND_2;

typedef struct
{
    unsigned int command  :2;    /* COMMAND BITS --- SHOULD BE SET TO 11*/
    unsigned int highpass :1;    /* HIGHPASS FILTER ENABLE              */
    unsigned int loopback :1;    /* LOOPBACK TEST ENABLE                */
    unsigned int aux      :1;    /* AUX INPUT ENABLE                    */
    unsigned int sync     :1;    /* SYNCHRONOUS TRANSMIT/RECEIVE ENABLE */
    unsigned int gain     :2;    /* GAIN SELECTION BITS                 */
    unsigned int d_8      :1;    /* UNUSED - DON'T CARES                */
    unsigned int sinx	  :1;	 /* SINX/X CORRECTION FILTER ENABLE	*/
#if TLC32046
    unsigned int d10out   :1;	 /* OUTPUT TO D10OUT (TEL_I/F MODE)	*/
    unsigned int d11out   :1;	 /* OUTPUT TO D11OUT (TEL_I/F MODE)	*/
    unsigned int d_cdef   :4;	 /* UNUSED - DON'T CARES                */
#else
    unsigned int d_abcdef :6;	 /* UNUSED - DON'T CARES                */
#endif

} AIC_COMMAND_3;

/*========================================================================*/
/* GLOBAL VARIABLES *=====================================================*/
/*========================================================================*/
extern int t_buffer;          /* SIZE OF I/O BUFFER(S)                   */
extern int r_buffer;          /* SIZE OF I/O BUFFER(S)                   */
extern VPVF output;           /* OUTPUT DATA BUFFER FOR PROCESSOR        */
extern VPVF input;            /* INPUT DATA BUFFER FOR PROCESSOR         */
extern VPVF output_xfer;      /* OUTPUT DATA BUFFER FOR ISR/AIC          */
extern VPVF input_xfer;       /* INPUT DATA BUFFER FOR ISR/AIC           */
extern VI   buffer_rcvd;      /* CPU-ISR COMM FLAG (INPUT)               */
extern VI   buffer_xmtd;      /* CPU-ISR COMM FLAG (OUTPUT)              */
extern VI   r_index;          /* INDEX INTO INPUT AND OUTPUT DATA ARRAYS */
extern VI   t_index;          /* INDEX INTO INPUT AND OUTPUT DATA ARRAYS */
extern VI   i;                /* GENERIC COUNTER VARIABLE                */

/*------------------------------------------------------------------------*/
/* AIC CONTROL VARIABLES						  */
/*------------------------------------------------------------------------*/
extern AIC_COMMAND_0  aic_command_0;                /* AIC COMMAND WORD 0 */
extern AIC_COMMAND_1  aic_command_1;                /* AIC COMMAND WORD 1 */
extern AIC_COMMAND_2  aic_command_2;                /* AIC COMMAND WORD 2 */
extern AIC_COMMAND_3  aic_command_3;                /* AIC COMMAND WORD 3 */
extern volatile AIC_PRIMARY    aic_primary;
extern VI secondary_transmit;          /* FLAG TO SENT SECONDARY TRANSMIT*/
extern VI aic_secondary;             /* COMMAND SENT ON SECONDARY TRANSMIT */
extern int ie;                        /* ENABLED INTERRUPTS TEMP VARIABLE */

/**************************************************************************/
/*	   FUNCTION PROTOTYPES						  */
/**************************************************************************/
/************************/
/* AIC Driver functions */
/************************/
void init_arrays(int t_buffer, int r_buffer);
void init_aic(void);
void configure_aic(int i);
#if SER_NUM
void c_int07(void);
#if ASYNC
void c_int08(void);
#endif
#else
void c_int05(void);
#if ASYNC
void c_int06(void);
#endif
#endif

/*************************/
/* Generic C30 functions */
/*************************/
void put_ie(int);
int  get_ie(void);
